function varargout = fem(varargin)

% FEM Application M-file for fem.fig
%    FIG = FEM launch fem GUI.
%    FEM('callback_name', ...) invoke the named callback.

% =========================================================================
% Author: Anton Zaicenco <a.zaicenco@codedevelopment.net>
% Version:          2.1.2
% Date (started):   Dec 2004
% Latest update:    Nov 08, 2014
% =========================================================================

% -------------------------------------------------------------------------
% "handles" stores the following additional data:
%       handles.in_data   -  structure (all data about the object)
%                    handles.in_data.dofN - dof's per node
%                    handles.in_data.type:
%                                  '2dbeam', '3dtrus', '3dbeam', 'cstcsq',
%                                  '3dbrik', 'bciz  ', 'tetrah'
% -------------------------------------------------------------------------

if nargin == 0 
    % Generate a structure of handles to pass to callbacks, and store it
	fig = openfig(mfilename,'reuse');
	handles = guihandles(fig);
	guidata(fig, handles);
	if nargout > 0
		varargout{1} = fig;
	end
elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
	try
		[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
	catch
		disp(lasterr);
	end
end


% -------------------------------------------------------------------------
function varargout = loaddata_Callback(h, eventdata, handles, varargin)

dPath = which ('inpfiles.txt');
cd (dPath(1:end-12));
!time /t > inpfiles.txt

set(handles.text3, 'ForegroundColor',[0 0 0]);
set(handles.text4, 'ForegroundColor',[0 0 0]);
set(handles.text12,'ForegroundColor',[0 0 0]);

[nf,cf] = uigetfile('*.m',' Load input data ');

if length(cf)>1
    set(handles.text7,'string',[cf ' ' nf]);
end;

for i=1:length(nf)
    if nf(i)~='.' AA(i)=nf(i); end;
    if nf(i)=='.' break; end;
end;
if nf~=0
    handles.run.objBuilt = 0;
    set(handles.pushbutton2,'ForegroundColor','k');
    set( handles.popupmenu3,'Value', 1 );
    set( handles.popupmenu3,'String', '1' );
    handles.run.ModeShow.OK = 0; 
    set(handles.pushbutton12,'ForegroundColor','k');
    handles.filename = AA;
    handles.in_data = eval (AA); 
    in_data = handles.in_data;
    if  isfield(handles,'shownodes')
        if handles.shownodes==1 
            fem_0_plotGUIdata(in_data,1);
        else fem_0_plotGUIdata(in_data,0);
        end
    else fem_0_plotGUIdata(in_data,0);
    end;
    format long;
    set(handles.text3,'string',[ num2str(size(in_data.ND,1))]);
    set(handles.text3,'ForegroundColor',[1 0 0]);
    set(handles.text12,'string',[ num2str(size(in_data.EL,1))]);
    set(handles.text12,'ForegroundColor',[1 0 0]);
    
    [dofN,EL_TYPE] = fem_0_elemType (in_data);
    switch EL_TYPE
        case {0,1,2}
            handles.in_data.dofN = dofN; handles.in_data.type = '2dbeam';
        case {31}
            handles.in_data.dofN = dofN; handles.in_data.type = '3dtrus';
        case {3}
            handles.in_data.dofN = dofN; handles.in_data.type = '3dbeam';
        case {4,5,51}
            handles.in_data.dofN = dofN; handles.in_data.type = 'cstcsq';
        case {6}
            handles.in_data.dofN = dofN; handles.in_data.type = '3dbrik';
        case {9}
            handles.in_data.dofN = dofN; handles.in_data.type = 'bciz  ';
        case {10}
            handles.in_data.dofN = dofN; handles.in_data.type = 'tetrah';
    end

    set(handles.text4,'string',[ num2str(dofN*size(in_data.ND,1))]); % DOFs
    set(handles.text4,'ForegroundColor',[1 0 0]);
    format;
end
guidata(h, handles); 


% --------------------------------------------------------------------
function varargout = visualize_Callback(h, eventdata, handles, varargin)

if  isfield(handles,'run')
    if  isfield(handles.run,'animate')
        if  isfield(handles.run.animate,'ok')
            if handles.run.animate.ok==1
                in_data = handles.in_data;
                if isfield(handles,'resp')
                    resp = handles.resp;
                end
                fem_0_dynAnim(resp,in_data);
            end
        end
    end
end

guidata(h, handles);

% --------------------------------------------------------------------
function varargout = checkbox1_Callback(h, eventdata, handles, varargin)
if (get(h,'Value') == get(h,'Max'))
    handles.run.static=1;
    handles.ta(1)=1;
else
    handles.run.static=0;
    handles.ta(1)=0;
end
guidata(h, handles);



% --------------------------------------------------------------------
function varargout = checkbox2_Callback(h, eventdata, handles, varargin)
if (get(h,'Value') == get(h,'Max'))
    handles.run.dynamic=1;
    handles.ta(2)=1;
else
    handles.run.dynamic=0;
    handles.ta(2)=0;
end
guidata(h, handles);



% --------------------------------------------------------------------
function varargout = checkbox3_Callback(h, eventdata, handles, varargin)
if (get(h,'Value') == get(h,'Max'))
    handles.run.modal=1;
    handles.ta(3)=1;
else
    handles.run.modal=0;
    handles.ta(3)=0;
end
guidata(h, handles);



% --------------------------------------------------------------------
function varargout = pushbutton2_Callback(h, eventdata, handles, varargin)
handles.ta = [0 0 0];

if  isfield(handles,'run') & isfield(handles,'in_data')
  if isfield(handles.run,'objBuilt')
    if (handles.run.objBuilt == 1)
      if  isfield(handles.run,'static') 
          if handles.run.static==1
              in_data = handles.in_data;
              handles.ta(1) = 1;
              showPS = 0;
              if  isfield(handles,'show')
                  if handles.show.PS==1 showPS=1; end;
                  if handles.show.PS==0 showPS=0; end;
              end
          end;
      end
      if  isfield(handles.run,'dynamic') 
          if handles.run.dynamic==1
              handles.ta(2) = 1; showPS = 1;
              dofN = handles.in_data.dofN;
              pax = gca;
              fem_0_plotPt(handles,handles.in_data,pax,1);
              handles.run.animate.ok=1;
          end
      end
      if  isfield(handles.run,'modal')
          if handles.run.modal==1  handles.ta(3) = 1; showPS = 1;  end;
      end
      if length(find(handles.ta==1))>0  
          [handles.obj,handles.resp] = ...
              fem_0_run(handles.in_data,handles.ta,handles.obj, ...
                        handles.L,handles.break_K,showPS);
      end
    end
  end
end;         
guidata(h, handles);

% --------------------------------------------------------------------
function varargout=pushbutton1_ButtondownFcn(h,eventdata,handles,varargin)
% Stub for ButtondownFcn of the uicontrol handles.pushbutton1.
%disp('pushbutton1 ButtondownFcn not implemented yet.')


% --------------------------------------------------------------------
function varargout = checkbox1_ButtondownFcn(h, eventdata, handles, varargin)
% Stub for ButtondownFcn of the uicontrol handles.checkbox1.
%disp('checkbox1 ButtondownFcn not implemented yet.')


% --------------------------------------------------------------------
function varargout = radiobutton2_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.radiobutton2.
%disp('radiobutton2 Callback not implemented yet.')

% --------------------------------------------------------------------
function varargout = radiobutton3_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.radiobutton3.
%disp('radiobutton3 Callback not implemented yet.')

% --------------------------------------------------------------------
function varargout = pushbutton2_ButtondownFcn(h, eventdata, handles, varargin)
% Stub for ButtondownFcn of the uicontrol handles.pushbutton2.
%disp('pushbutton2 ButtondownFcn not implemented yet.')

% --------------------------------------------------------------------
function varargout = pushbutton5_Callback(h, eventdata, handles, varargin)
if isfield(handles,'figure1') & ishandle(handles.figure1),
    close(handles.figure1);
end
% Stub for Callback of the uicontrol handles.pushbutton5.
sp_=sprintf('\n');
disp([sp_ '  Thank you for using FEM toolbox' sp_])

% --------------------------------------------------------------------
function varargout = edit1_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.edit1.
%disp('edit1 Callback not implemented yet.')

% --------------------------------------------------------------------
function varargout = text3_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.text3.
%disp('text3 Callback not implemented yet.')


% --------------------------------------------------------------------
function varargout = checkbox8_Callback(h, eventdata, handles, varargin)
if (get(h,'Value') == get(h,'Max'))
    handles.shownodes=1;
else
    handles.shownodes=0;
end
guidata(h, handles);
% Stub for Callback of the uicontrol handles.checkbox8.



% --------------------------------------------------------------------
function varargout = pushbutton8_Callback(h, eventdata, handles, varargin)
saveOBJECTeps; 
% Stub for Callback of the uicontrol handles.pushbutton8.



% --------------------------------------------------------------------
function varargout = checkbox9_Callback(h, eventdata, handles, varargin)
if (get(h,'Value') == get(h,'Max'))
    handles.show.PS=1;
else
    handles.show.PS=0;
end
guidata(h, handles);



% --------------------------------------------------------------------
function varargout = figure1_CreateFcn(h, eventdata, handles, varargin)
% Stub for CreateFcn of the figure handles.figure1.
% disp('figure1 CreateFcn not implemented yet.')

% --------------------------------------------------------------------
function pushbutton11_Callback(hObject, eventdata, handles)
if  isfield(handles,'filename')
    if length(handles.filename)>0
       type_analysis = [1 1 1];
       [handles.obj,handles.L,handles.break_K] = fem_0_mainKM (handles.in_data,type_analysis);
       handles.run.objBuilt = 1;
       handles.run.ModeShow.OK = 1;
       Str_ = cell(min(size(handles.obj.M,1),10), 1);
       for i=1:min(size(handles.obj.M,1),20)
           Str_{i} = num2str(i);
       end
       set( handles.popupmenu3, 'String', Str_ );
       set(handles.pushbutton12,'ForegroundColor','red');
       set(handles.pushbutton2,'ForegroundColor','red');
       % ------
       guidata(hObject, handles);
    end
end



function pushbutton12_Callback(hObject, eventdata, handles)

if  isfield(handles,'run')
    if  isfield(handles.run,'ModeShow')
        if  isfield(handles.run.ModeShow,'OK')
            if handles.run.ModeShow.OK==1
                if  isfield(handles.run,'modal')
                    showLines = 0; 
                    if handles.run.modal==1
                        if (get(handles.checkbox10,'Value')==1)
                            showLines = 1;
                        end
                        stringData = get(handles.popupmenu3,'String');
                        modeN = stringData{get( handles.popupmenu3,'Value')};
                        fem_2_modal(handles.in_data,handles.obj, ...
                            handles.L,str2num(modeN),showLines,1000210);
                    end;
                end
            end
        end
    end
end




% --- Executes during object creation, after setting all properties.
function popupmenu3_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in checkbox10.
function checkbox10_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox10 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox10


% --- Executes on selection change in popupmenu3.
function popupmenu3_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popupmenu3 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu3


