function [xc, t] = fem_2_dyn(in_data, obj, L, LK) 

% -------------------------------------------------------------------------


global dof h CH CH2 C K zdd delta_t

dimn = length(L);

[dof_per_node] = fem_0_elemType(in_data);

dof = size(in_data.ND,1)*dof_per_node;
hh  = zeros(dof(1),1);


for i=1:length(in_data.dynam.TIMEHM)
    t4 = in_data.dynam.TIMEHM(i)*dof_per_node-(dof_per_node-1);
    for k=1:dof_per_node
        hh(t4+(k-1))=1;
    end;
end;

for i=1:length(obj.M)
   if obj.M(i,i)>0
      if hh(i)==1  hh(i)= 1;  else hh(i)= 0; end;
   end;
end;

for i=1:dof_per_node:dof(1)
    for k=1:dof_per_node
        if hh(i+(k-1))~=0  & in_data.dynam.TIMEHDIR(k)==0  
            hh(i+(k-1))=0;   end;
        if hh(i+(k-1))~=0  & in_data.dynam.TIMEHDIR(k)~=0  
            hh(i+(k-1))=hh(i+(k-1))*in_data.dynam.TIMEHDIR(k); end;
    end;
end;


if  isfield(in_data,'slaves')
    if ~isempty(in_data.slaves)
        [u,T] = fem_2_staticMslave(obj.Ksys.Kgl,L,123,in_data.slaves, ...
                in_data.master,0);
        obj.M_old = obj.M;
        clear u
        obj.M = T'*obj.M*T;
        K = T'*obj.Ksys.Kgl*T;
        hh = T'*hh;
        h = hh;
    end
else
    h=hh(L);
end

clear hh

zdd     = load(in_data.dynam.TIMEH);
points  = length(zdd);
delta_t = in_data.dynam.delta_tm;

disp(['     Dynamic analysis - start Cholesky factorization. Time (sec): ' num2str(toc) ]);

if  isfield(in_data,'slaves')
    if length(in_data.slaves)>0
        MM = obj.M;
    end
else
    MM = obj.M(L,L);
end


switch 2
    case 1
        damping = in_data.dynam.DAMP_C;
        N=in_data.dynam.DAMP_F;
        if  isfield(in_data,'slaves')
            if ~isempty(in_data.slaves)
                [alp, bet] = fem_3_dynRayleigh(obj.Ksys.Kgl, ...
                    obj.M,damping,N);
            end
        else
            [alp, bet] = fem_3_dynRayleigh(obj.Ksys.Kgl(L,L), ...
                obj.M(L,L),damping,N);
        end
    case 2
        alp = in_data.dynam.ab(1);
        bet = in_data.dynam.ab(2);
end

disp([' Rayleigh coef.: Alpha =  ' num2str(alp) ' . Beta = ' num2str(bet)]);

if  isfield(in_data,'slaves')
    if ~isempty(in_data.slaves)
        C = alp*obj.M + bet*K; 
    end
else
    K = obj.Ksys.Kgl(L,L);
    C = alp*obj.M(L,L) + bet*obj.Ksys.Kgl(L,L);
end


clear i k t4 damping
dof = length(C);
x0 = zeros(2*dof,1);
t      = [1:points-1]*delta_t;
disp(['     Dynamic analysis - START. Time (sec): ' num2str(toc) ]);

alp = 0.2;
bet = ((1+alp)^2)/4;
gam = 1/2 + alp;
displ = zeros(length(zdd)-1,dof);
veloc = zeros(length(zdd)-1,dof);
accel = zeros(length(zdd)-1,dof);
Z = ((MM+delta_t.*(1-alp)*gam*C + delta_t^2*(1-alp)*bet*K));
Z1= (delta_t*(1-alp)*(1-gam)*C+delta_t^2*(1-alp)*(0.5-bet)*K);
Z2= (C+delta_t*(1-alp)*K);

for i=2:length(zdd)-1
    accel(i,:) = Z \ ( - Z1*accel(i-1,:)' - Z2*veloc(i-1,:)' - ...
        K*displ(i-1,:)' -(1-alp)*MM*h*zdd(i) -alp*MM*h*zdd(i-1));
    veloc(i,:) = veloc(i-1,:) + delta_t*((1-gam)*accel(i-1,:) + ...
        gam*accel(i,:) );
    displ(i,:) = displ(i-1,:) + delta_t*veloc(i-1,:) + ...
        delta_t^2*((0.5-bet)*accel(i-1,:)+bet*accel(i,:) );
end

xc = zeros(length(zdd)-1,2*dof);
xc(:,1:dof) = displ;
xc(:,dof+1:end) = veloc;

clear displ veloc

disp(['     Dynamic analysis - END.   Time (sec): ' num2str(toc) ])
clear CH  CH2 h zdd x0 K

disp(['     Dynamic analysis : recovering displ., veloc., and accel' ])

sd   = length(t);
h=[1:LK]; h(L)=0;

if  isfield(in_data,'slaves')
    if ~isempty(in_data.slaves)
        xc_d = T*xc(:,1:size(T,2))';
        xc_v = T*xc(:,size(T,2)+1:end)';
        xc_a = T*accel';
        xc   = [xc_d' xc_v' xc_a'];
        dof  = length(L);
        C    = alp*obj.M_old(L,L) + bet*obj.Ksys.Kgl(L,L);
    end
else
    xc   = [xc accel];
    xc   = [ xc   zeros(sd,3*(LK-dof)) ];
    xc(:,2*LK+L) = xc(:,2*dof+1:3*dof);
    xc(:,LK+L) = xc(:,dof+1:2*dof);
    xc(:,L)    = xc(:,1:dof);
    xc(:,find(h))=0; xc(:,LK+find(h))=0;
    xc(:,2*LK+find(h))=0;
end

disp(['     Dynamic analysis : velocities and displacements recovered' ])
clear accel

clear h

time_now = toc;
time_min = floor(time_now/60); time_hrs = floor(time_min/60);
time_min = floor(time_min-time_hrs*60);
time_sec = floor(time_now-time_hrs*60*60-time_min*60);

disp([     '  ...   Dynamic analysis: OK.        Time:  ' ...
    num2str(time_hrs) ' hrs : ' num2str(time_min) ' min : ' ...
    num2str(time_sec) ' sec ' ])


